import unicodedata
from pathlib import Path
from .agl import AGL


def __parse_file(filename):
    file = Path(__file__).parent / f'agl-aglfn/{filename}.txt'
    lines = file.read_text().splitlines()
    return {unicodedata.lookup(_.split(';')[2]): _.split(';') for _ in lines if not _.startswith('#')}


__db = __parse_file('aglfn')


def __glyphs():
    return list(__db.keys())


def to_glyph(aglfn):
    result = [k for k, v in __db.items() if v[1] == aglfn]
    return result[0] if result else None


def __names():
    return [v[1] for k, v in __db.items()]


def name(glyph):
    result = __db.get(glyph)
    return result[1] if result else None


names = __names()
glyphs = __glyphs()
__all__ = ['name', 'names', 'glyphs', 'to_glyph', 'AGL']



import unicodedata
from pathlib import Path
from functools import lru_cache


class AGL:
    def __init__(self):
        self.glyph_list = self.__parse_file('glyphlist')

    @staticmethod
    def __parse_file(name):
        file = Path(__file__).parent / f'agl-aglfn/{name}.txt'
        return file.read_text().splitlines()

    @lru_cache(maxsize=2)
    def glyph_names(self):
        result = {}
        for line in self.glyph_list:
            if '#' in line: continue
            value, key = line.split(";")
            result.setdefault(key, []).append(value)
        return result

    @lru_cache(maxsize=2)
    def all_glyphs(self):
        return [self.__glyph_name(ucp) for ucp in range(0, 0x10FFFF+1)]

    @staticmethod
    def char_to_usv(char: str):
        return hex(ord(char))[2:].upper().zfill(4)

    @staticmethod
    def char_to_ucp(char: str):
        return ord(char)

    def char_to_glyph_names(self, char: str):
        return self.__glyph_name(self.char_to_ucp(char))

    # @staticmethod
    # def usv_to_ucp(usv: str):
    #     return int(usv, 16)
    #
    # def usv_to_char(self, usv: str):
    #     return chr(self.usv_to_ucp(usv))
    #
    # def usv_to_glyph_name(self, usv: str):
    #     return self.__glyph_name(self.usv_to_ucp(usv))

    @staticmethod
    def ucp_to_str(ucp: int):
        return hex(ucp)[2:].upper().zfill(4)

    def __glyph_name(self, ucp: int):
        return self.glyph_names().get(self.ucp_to_str(ucp))

    @lru_cache(maxsize=0)
    def lookup(self, char: str):
        return dict(
            char=char,
            unicode_code_point=self.char_to_ucp(char),
            unicode_standard_value=self.char_to_usv(char),
            agl_glyph_names=self.char_to_glyph_names(char),
            unicode_character_name=unicodedata.name(char, None)
        )
